from sqlalchemy import Column, Integer, String, Float, Boolean, ForeignKey, Date, DateTime
from sqlalchemy.orm import relationship

from univer_db.orm import get_base


Base = get_base()


class StructureDivision(Base):
    """
    Модель "Подразделение"
    """

    __tablename__ = 'univer_structure_division_1c'

    # Идентификатор
    id = Column('structure_division_id', Integer, primary_key=True)

    # Наименование
    name_kz = Column('structure_division_name_kz', String(500))
    name_ru = Column('structure_division_name_ru', String(500))
    name_en = Column('structure_division_name_en', String(500))

    # Родительское подразделение
    parent_id = Column('structure_division_ext', ForeignKey(
        'univer_structure_division_1c.structure_division_id'))
    parent = relationship('StructureDivision',
                          remote_side='StructureDivision.id')

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<StructureDivision {}>'.format(self)

    def __str__(self):
        return self.name_ru


class PersonnelType(Base):
    """
    Тип сотрудника
    """
    __tablename__ = 'univer_type_personal_1c'

    # Идентификатор
    id = Column('type_personal_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('type_personal_name_ru', String(100))
    name_kz = Column('type_personal_name_kz', String(100))
    name_en = Column('type_personal_name_en', String(100))

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<PersonnelType {} (id={} status={})'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class PersonnelStructureDivisionLink(Base):
    """
    Связь между сотрудником и подразделением
    """
    __tablename__ = 'univer_personal_struct_pos_link_1c'

    # Идентификатор
    id = Column('pers_struct_pos_link_id', Integer, primary_key=True)

    # Статус
    status = Column(Integer)

    # Сотрудник
    personnel_id = Column('personal_id', ForeignKey(
        'univer_personal.personal_id'))
    personnel = relationship('Personnel')

    # Подразделение
    structure_division_id = Column(ForeignKey(
        'univer_structure_division_1c.structure_division_id'))
    structure_division = relationship('StructureDivision')

    # Должность сотрудника
    personnel_position_id = Column('personal_position_id', ForeignKey(
        'univer_personal_position_1c.personal_position_id'))
    personnel_position = relationship('PersonnelPosition')

    # Тип сотрудника
    personnel_type_id = Column('type_id', ForeignKey(
        'univer_type_personal_1c.type_personal_id'))
    personnel_type = relationship('PersonnelType')

    # Ставка
    rate = Column('personal_rate', Float)

    def __repr__(self):
        return '<PersonnelStructureDivisionLink {}>'.format(self)

    def __str__(self):
        return '{} ({})'.format(self.personnel, self.structure_division)


class Faculty(Base):
    """
    Модель "Факультет"
    """

    __tablename__ = 'univer_faculty'

    id = Column('faculty_id', Integer, primary_key=True)
    status = Column(Integer)
    name_kz = Column('faculty_name_kz', String(200))
    name_ru = Column('faculty_name_ru', String(200))
    name_en = Column('faculty_name_en', String(200))

    def __repr__(self):
        return "<Faculty {}>".format(self)

    def __str__(self):
        return self.name_ru


class Chair(Base):
    """
    Модель "Кафедра"
    """

    __tablename__ = 'univer_chair'

    id = Column('chair_id', Integer, primary_key=True)
    faculty_id = Column(ForeignKey('univer_faculty.faculty_id'))
    faculty = relationship('Faculty')
    status = Column(Integer)
    name_kz = Column('chair_name_kz', String(200))
    name_ru = Column('chair_name_ru', String(200))
    name_en = Column('chair_name_en', String(200))
    structure_division_id = Column(ForeignKey(
        'univer_structure_division_1c.structure_division_id'))
    structure_division = relationship('StructureDivision')

    def __repr__(self):
        return '<Chair {}>'.format(self)

    def __str__(self):
        return self.name_ru


class PersonnelPosition(Base):
    """
    Модель "Должность сотрудника"
    """
    __tablename__ = 'univer_personal_position_1c'

    # Идентификатор
    id = Column('personal_position_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('personal_position_name_ru', String(500))
    name_kz = Column('personal_position_name_kz', String(500))
    name_en = Column('personal_position_name_en', String(500))

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<PersonnelPosition {}>'.format(self)

    def __str__(self):
        return self.name_ru



import unittest
from univer_db.models import StructureDivision

from . import Session


class TestStructureDivision(unittest.TestCase):
    def test_get_obj(self):
        session = Session()
        division = session.query(StructureDivision).first()
        print("division: %s" % division)

    def test_get_parent(self):
        session = Session()
        division = session.query(StructureDivision).filter(
            StructureDivision.parent_id != 0).first()
        print("division: %s" % division)
        print("division.parent_id: %s" % division.parent_id)
        print("division.parent: %s" % division.parent)



from sqlalchemy import Column, Integer, String, Float, Boolean, ForeignKey, Date, DateTime
from sqlalchemy.orm import relationship, validates

from univer_db.orm import get_base


Base = get_base()


class Speciality(Base):
    """
    Модель "Специальность"
    """

    __tablename__ = 'univer_speciality'
    __table_args__ = {'extend_existing': True}

    id = Column('speciality_id', Integer, primary_key=True)
    status = Column(Integer)
    faculty_id = Column(ForeignKey('univer_faculty.faculty_id'))
    faculty = relationship('Faculty')
    stage_id = Column(ForeignKey('univer_stage.stage_id'))
    stage = relationship('Stage')
    name_kz = Column('speciality_name_kz', String(200))
    name_ru = Column('speciality_name_ru', String(200))
    name_en = Column('speciality_name_en', String(200))
    code = Column('speciality_okpd', String(10))
    description_kz = Column('speciality_discription_kz', String)
    description_ru = Column('speciality_discription_ru', String)
    description_en = Column('speciality_discription_en', String)
    result_ru = Column(String)
    result_kz = Column(String)
    result_en = Column(String)
    type = Column(Integer)

    def __repr__(self):
        return "<Speciality {}>".format(self)

    def __str__(self):
        return "{} - {}".format(self.code, self.name_ru)


class SpecialityChair(Base):
    """
    Модель отношений "Специальность-Кафедра"
    """

    __tablename__ = 'univer_speciality_chair'
    __table_args__ = {'extend_existing': True}

    speciality_id = Column(ForeignKey(
        'univer_speciality.speciality_id'), primary_key=True)
    speciality = relationship('Speciality')
    chair_id = Column(ForeignKey('univer_chair.chair_id'), primary_key=True)
    chair = relationship('Chair')

    def __repr__(self):
        return '<SpecialityChair {}>'.format(self)

    def __str__(self):
        return '{} - {}'.format(self.chair, self.speciality)


class Subject(Base):
    """
    Дисциплина
    """
    __tablename__ = 'univer_subject'

    # Идентификатор
    id = Column('subject_id', Integer, primary_key=True)

    # Тип дисциплины
    # 0 - Обязательная дисциплина
    # 1 - Элективная дисциплина
    # 2 - Практика
    # 3 - Спец. дисциплина
    type = Column('subject_type', Integer)

    # Наименование
    name_kz = Column('subject_name_kz', String(500))
    name_ru = Column('subject_name_ru', String(500))
    name_en = Column('subject_name_en', String(500))

    # Описание
    description_kz = Column('subject_description_kz', String)
    description_ru = Column('subject_description_ru', String)
    description_en = Column('subject_description_en', String)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return f'<Subject {self} (id={self.id} status={self.status} type={self.type})>'

    def __str__(self):
        return self.name_ru


class EducType(Base):
    """
    Тип обучения
    """
    __tablename__ = 'univer_educ_type'

    # Идентификатор
    id = Column('educ_type_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('educ_type_name_ru', String(100))
    name_kz = Column('educ_type_name_kz', String(100))
    name_en = Column('educ_type_name_en', String(100))

    def __repr__(self):
        return '<EducType {}>'.format(self.name_ru)

    def __str__(self):
        return self.name_ru


class LangDivision(Base):
    """
    Модель "Языковый отдел"
    """
    __tablename__ = 'univer_lang_division'

    # Идентификатор
    id = Column('lang_division_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('lang_division_name_ru', String(100))
    name_kz = Column('lang_division_name_kz', String(100))
    name_en = Column('lang_division_name_en', String(100))

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<LangDivision {} (id={}, status={})>'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class DocumentIdentity(Base):
    """
    Тип документа
    """
    __tablename__ = 'univer_document_identity'

    id = Column('document_identity_type', Integer, primary_key=True)
    name_ru = Column('document_name_ru', String(100))
    name_kz = Column('document_name_kz', String(100))
    name_en = Column('document_name_en', String(100))

    def __repr__(self):
        return '<DocumentIdentity {} (id={})'.format(self, self.id)

    def __str__(self):
        return self.name_ru


class Institution(Base):
    """
    Модель "Учебное заведение"
    Статус: Выполняется
    """

    __tablename__ = 'univer_edu_institutions'

    id = Column('edu_institution_id', Integer, primary_key=True)
    name_kz = Column('edu_institution_name_kz', String)
    name_ru = Column('edu_institution_name_ru', String)
    name_en = Column('edu_institution_name_en', String)

    def __repr__(self):
        return '<Institution {} (id={})>'.format(self, self.id)

    def __str__(self):
        return self.name_ru


class GraduateDocType(Base):
    """
    Тип документа об окончании учебного заведения перед поступлением в университет
    Статус: Выполняется
    """

    __tablename__ = 'univer_graduate_doctypes'

    id = Column('graduate_doctype_id', Integer, primary_key=True)
    name_ru = Column('graduate_doctype_name_ru', String)
    name_kz = Column('graduate_doctype_name_kz', String)
    name_en = Column('graduate_doctype_name_en', String)

    def __repr__(self):
        return '<GraduateDocType {} (id={})>'.format(self, self.id)

    def __str__(self):
        return self.name_ru


class GraduateInfo(Base):
    """
    Данные об окончании учебного заведения перед поступлением в университет
    Статус: Выполняется
    """

    __tablename__ = 'univer_graduate_info'

    id = Column('graduate_info_id', Integer, primary_key=True)
    date = Column('graduate_info_date', DateTime)
    institution_name = Column('graduate_info_institution_name', String)
    series = Column('graduate_info_series', String)
    number = Column('graduate_info_number', String)
    graduate_doctype_id = Column(ForeignKey(
        'univer_graduate_doctypes.graduate_doctype_id'))
    graduate_doctype = relationship('GraduateDocType')
    institution_id = Column('edu_institution_id', ForeignKey(
        'univer_edu_institutions.edu_institution_id'))
    institution = relationship('Institution')

    def __repr__(self):
        return '<Graduate {} (id={})>'.format(self, self.id)

    def __str__(self):
        return '{} {}'.format(self.series, self.number)


class AcademCalendar(Base):
    """
    Модель "Академический календарь"
    """

    __tablename__ = 'univer_academ_calendar_pos'

    id = Column('acpos_id', Integer, primary_key=True)
    educ_plan_id = Column(ForeignKey('univer_educ_plan.educ_plan_id'))
    educ_plan = relationship('EducPlan')
    acpos_semester = Column(Integer)
    acpos_module = Column(Integer)
    controll_id = Column('control_id', ForeignKey('univer_control.control_id'))
    controll = relationship('Controll')
    acpos_weeks = Column(Integer)
    acpos_date_start = Column(DateTime)
    acpos_date_end = Column(DateTime)

    def __repr__(self):
        return '<AcademCalendar {}>'.format(self)

    def __str__(self):
        return '{}'.format(self.id)


class EducPlan(Base):
    """
    Учебный план
    """
    __tablename__ = 'univer_educ_plan'

    # Идентификатор
    id = Column('educ_plan_id', Integer, primary_key=True)

    # Специальность
    speciality_id = Column('speciality_id', ForeignKey(
        'univer_speciality.speciality_id'))
    speciality = relationship('Speciality')

    # Форма обучения
    education_form_id = Column(ForeignKey(
        'univer_education_form.education_form_id'))
    education_form = relationship('EducationForm')

    # Уровень образования
    edu_level_id = Column(ForeignKey('univer_edu_levels.edu_level_id'))
    edu_level = relationship('EduLevel')

    # Учебный год
    year = Column('educ_plan_adm_year', Integer)

    # Количество семестров в учебном плане
    semesters = Column('educ_plan_number_of_semestr', Integer)

    # Количество обязательных кредитов
    min_cred_total = Column('educ_plan_min_cred_total', Integer)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return "<EducPlan {} {}>".format(self.speciality, self.year)

    def __str__(self):
        return "{} {}".format(self.speciality, self.year)


class EducPlanPos(Base):
    """
    Позиция учебного плана
    """
    __tablename__ = 'univer_educ_plan_pos'

    # Идентификатор
    id = Column('educ_plan_pos_id', Integer, primary_key=True)

    # Учебный план
    educ_plan_id = Column('educ_plan_id', ForeignKey(
        'univer_educ_plan.educ_plan_id'))
    educ_plan = relationship('EducPlan')

    # Код по РУП
    code = Column('rup_ru', String(50))

    # Дисциплина
    subject_id = Column('subject_id', ForeignKey('univer_subject.subject_id'))
    subject = relationship('Subject')

    # Тип контроля
    controll_type_id = Column(ForeignKey(
        'univer_controll_type.controll_type_id'))
    controll_type = relationship('ControllType')

    # Количество кредитов
    credit = Column('educ_plan_pos_credit', Integer)

    # Семестр, к котороу привязана позиция учебного плана
    semester = Column('educ_plan_pos_semestr', Integer)

    # Цикл
    cycle_id = Column(ForeignKey('univer_cycles.cycle_id'))
    cycle = relationship('Cycle')

    # Цикловой компонент
    cycle_component_id = Column(ForeignKey(
        'univer_cycle_component.cycle_component_id'))
    cycle_component = relationship('CycleComponent')

    # Количество часов
    hours = Column('educ_plan_pos_hours', Integer)

    # Статус
    status = Column(Integer)

    # Группа позиции учебного плана
    pos_group_id = Column(ForeignKey(
        'univer_educ_plan_pos_groups.pos_group_id'))
    pos_group = relationship('EducPlanPosGroup')

    def __repr__(self):
        return "<EducPlanPos {}: {} ({} семестр)>".format(self.educ_plan, self.subject, self.semester)

    def __str__(self):
        return "{}: {} ({} семестр)".format(self.educ_plan, self.subject, self.semester)


class Cycle(Base):
    """
    Модель "Цикл"
    """
    __tablename__ = 'univer_cycles'

    # Идентификатор
    id = Column('cycle_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('cycle_name_ru', String)
    name_kz = Column('cycle_name_kz', String)
    name_en = Column('cycle_name_en', String)

    def __repr__(self):
        return '<Cycle {}>'.format(self)

    def __str__(self):
        return self.name_ru


class CycleComponent(Base):
    """
    Цикловой компонент
    """
    __tablename__ = 'univer_cycle_component'

    # Идентификатор
    id = Column('cycle_component_id', Integer, primary_key=True)

    # Статус
    status = Column(Integer)

    # Наименование
    name_ru = Column('cycle_component_name_ru', String)
    name_kz = Column('cycle_component_name_kz', String)
    name_en = Column('cycle_component_name_en', String)
    short_name_ru = Column('cycle_component_short_name_ru', String)
    short_name_kz = Column('cycle_component_short_name_kz', String)
    short_name_en = Column('cycle_component_short_name_en', String)

    def __repr__(self):
        return f'<CycleComponent {self} (id={self.id} status={self.status})>'

    def __str__(self):
        return self.name_ru


class Attendance(Base):
    """
    Журнал посещений и успеваемости
    """
    __tablename__ = 'univer_attendance'

    # Дата
    date = Column('att_date', Date, primary_key=True)

    # Оценка/балл
    grade = Column('ball', Float)

    # Был на занятии
    was = Column(Boolean)

    @validates('was')
    def validate_was(self, key, value):
        """
        Поддерживает два значения:
        0 - не был
        1 - был
        """
        assert value in [0, 1]
        return value

    # Студент
    student_id = Column(ForeignKey(
        'univer_students.students_id'), primary_key=True)
    student = relationship('Student')

    # Группа студентов
    group_id = Column(ForeignKey('univer_group.group_id'), primary_key=True)
    group = relationship('Group')

    def __repr__(self):
        return f'<Attendance {self.student}: {self.grade} ({self.date})>'

    def __str__(self):
        return f'{self.student}: {self.grade} балл ({self.date})'


class EducLang(Base):
    """
    Модель "Язык обучения"
    """
    __tablename__ = 'univer_educ_lang'

    # Идентификатор
    id = Column('educ_lang_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('educ_lang_name_ru', String)
    name_kz = Column('educ_lang_name_kz', String)
    name_en = Column('educ_lang_name_en', String)

    # Короткое наименование
    short_name_ru = Column('educ_lang_short_name_ru', String)
    short_name_kz = Column('educ_lang_short_name_kz', String)
    short_name_en = Column('educ_lang_short_name_en', String)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<EducLang {}>'.format(self)

    def __str__(self):
        return self.name_ru


class Group(Base):
    """
    Дисциплинарная группа
    """
    __tablename__ = 'univer_group'

    # Идентификатор
    id = Column('group_id', Integer, primary_key=True)

    # Позиция учебного плана
    educ_plan_pos_id = Column('educ_plan_pos_id', ForeignKey(
        'univer_educ_plan_pos.educ_plan_pos_id'))
    educ_plan_pos = relationship('EducPlanPos', backref='groups')

    # Тип обучения
    educ_type_id = Column('educ_type_id', ForeignKey(
        'univer_educ_type.educ_type_id'))
    educ_type = relationship('EducType')

    # Преподаватель
    teacher_id = Column('teacher_id', ForeignKey('univer_teacher.teacher_id'))
    teacher = relationship('Teacher')

    # Языковый отдел
    lang_division_id = Column('lang_division_id', ForeignKey(
        'univer_lang_division.lang_division_id'))
    lang_division = relationship('LangDivision')

    # Академический год
    year = Column('group_year', Float)

    # Академический семестр
    semester = Column('group_semestr', Integer)

    # Минимальное/Максимальное количество студентов
    min_students = Column('group_students_min', Integer)
    max_students = Column('group_students_max', Integer)

    # Семестр повторного обучения
    retake_semester = Column('group_retake_semestr', Integer)

    # Модуль академического календаря
    acpos_module = Column('acpos_module', Integer)

    # Имя группы
    name = Column('group_name', String(40))

    # Язык обучения
    educ_lang_id = Column(ForeignKey('univer_educ_lang.educ_lang_id'))
    educ_lang = relationship('EducLang')

    # Кафедра
    chair_id = Column(ForeignKey('univer_chair.chair_id'))
    chair = relationship('Chair')

    def __repr__(self):
        return f'<Group {self} (id={self.id} educ_plan_pos_id={self.educ_plan_pos_id} teacher_id={self.teacher_id})>'

    def __str__(self):
        return f'{self.educ_plan_pos.educ_plan.speciality} ({self.educ_plan_pos.educ_plan.year} год)'


class GroupStudent(Base):
    """
    Студент в группе
    """
    __tablename__ = 'univer_group_student'

    # Идентификатор
    id = Column('group_student_id', Integer, primary_key=True)

    # Группа
    group_id = Column(ForeignKey('univer_group.group_id'))
    group = relationship('Group', backref='group_students')

    # Студент
    student_id = Column(ForeignKey('univer_students.students_id'))
    student = relationship('Student')

    # Дата и время выбора группы студентом
    choice_date = Column('student_choice_date', DateTime)

    # Идентификатор старого выбора группы студентом
    old_choice_id = Column('old_student_choice_id', Integer)

    # Студент зарегистрирован (Студент самостоятельно выбрал группу?)
    student_reg = Column(Boolean)

    # Неизвестное поле
    retake_type = Column(Integer)

    def __repr__(self):
        return f'<GroupStudent {self} (id={self.id} group={self.group_id} student={self.student_id})>'

    def __str__(self):
        return f'{self.group}: {self.student}'


class Controll(Base):
    """
    Контроль
    """
    __tablename__ = 'univer_control'

    # Идентификатор
    id = Column('control_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('control_name_ru', String(100))
    name_kz = Column('control_name_kz', String(100))
    name_en = Column('control_name_en', String(100))

    # Короткое наименование
    short_name_ru = Column('control_short_name_ru', String)
    short_name_kz = Column('control_short_name_kz', String)
    short_name_en = Column('control_short_name_en', String)

    # Максимальная оценка
    max_ball = Column('control_max_ball', Integer)

    # Статус
    status = Column(Integer)

    title = Column('control_title', String)
    has_sheet = Column('control_has_sheet', Integer)
    date_for_control_id = Column(Integer)
    is_exam = Column(Integer)

    def __repr__(self):
        return '<Controll {} (id={})>'.format(self, self.id)

    def __str__(self):
        return self.name_ru


class ControllType(Base):
    """
    Модель "Тип контроля"
    """

    __tablename__ = 'univer_controll_type'

    # Идентификатор
    id = Column('controll_type_id', Integer, primary_key=True)

    # Наименование
    name_kz = Column('controll_type_name_kz', String(100))
    name_ru = Column('controll_type_name_ru', String(100))
    name_en = Column('controll_type_name_en', String(100))

    def __repr__(self):
        return '<ControllType {}>'.format(self)

    def __str__(self):
        return self.name_ru


class ControllTypeControllLink(Base):
    """
    Модель "Связь между Controll и ControllType
    """
    __tablename__ = 'univer_controll_type_control_link'

    # Тип контроля
    controll_type_id = Column(ForeignKey(
        'univer_controll_type.controll_type_id'), primary_key=True)
    controll_type = relationship('ControllType')

    # Контроль
    controll_id = Column('control_id', ForeignKey(
        'univer_control.control_id'), primary_key=True)
    controll = relationship('Controll')

    # Тип ведомости
    sheet_type_id = Column(ForeignKey(
        'univer_sheet_type.sheet_type_id'), primary_key=True)
    sheet_type = relationship('SheetType')

    # Номер контроля
    control_number = Column(Integer)

    # Минимальный балл
    control_min_ball = Column(Integer)

    # Порция
    control_portion = Column(Float)

    retake = Column(Integer)

    def __repr__(self):
        return '<ControllTypeControllLink {} (controll_type_id={} controll_id={} sheet_type_id={}>'.format(self, self.controll_type_id, self.controll_id, self.sheet_type_id)

    def __str__(self):
        return '{}-{}'.format(self.controll_type, self.controll)


class MarkType(Base):
    """
    Тип оценки
    """
    __tablename__ = 'univer_mark_type'

    # Идентификатор
    id = Column('mark_type_id', Integer, primary_key=True)

    # Символ оценки
    symbol = Column('mark_type_symbol', String(10))

    # Минимальное значение
    min_val = Column('mark_type_minval', Integer)

    # Максимальное значение
    max_val = Column('mark_type_maxval', Integer)

    # GPA
    gpa = Column('mark_type_gpa', Float)

    # Наименование
    name_ru = Column('mark_text_ru', String(50))
    name_kz = Column('mark_text_kz', String(50))
    name_en = Column('mark_text_en', String(50))

    # Короткое наименование
    short_name_ru = Column('mark_text_short_ru', String(10))
    short_name_kz = Column('mark_text_short_kz', String(10))
    short_name_en = Column('mark_text_short_en', String(10))

    # Статус
    # Возможные значения:
    # - 1 (Активный)
    # - 2 (Архивный)
    status = Column('mark_status', Integer)

    @validates('status')
    def validate_status(self, key, value):
        """
        Значением status могут быть следующие значения:
        - 1 (Активный)
        - 2 (Архивный)
        """
        assert value in [1, 2]
        return value

    # Неизвестное поле
    arg = Column('mark_type_arg', Integer)

    # Неудаляемый
    # Эти типы оценок невозможно удалить через интерфейс Univer
    # Возможные значения:
    # - 0 (Можно удалить)
    # - 1 (Нельзя удалить)
    undeletable = Column(Integer)

    @validates('undeletable')
    def validate_undeletable(self, key, value):
        """
        Значением undeletable могут быть следующие значения:
        - 0 (Можно удалить)
        - 1 (Нельзя удалить)
        """
        assert value in [0, 1]
        return value

    # Неизвестное поле
    ects_symbol = Column('mark_type_ects_symbol', String(10))

    def __repr__(self):
        return f'<MarkType {self} (id={self.id} gpa={self.gpa} status={self.status})>'

    def __str__(self):
        return f'{self.name_ru}'


class Progress(Base):
    """
    Прогресс студента
    """
    __tablename__ = 'univer_progress'

    # Идентификатор
    id = Column('progress_id', Integer, primary_key=True)

    # Академический год
    academ_year = Column(Integer)

    # Академический семестр
    academ_semester = Column('semestr', Integer)

    # Студент
    student_id = Column(ForeignKey('univer_students.students_id'))
    student = relationship('Student')

    # Дисциплина
    subject_id = Column(ForeignKey('univer_subject.subject_id'))
    subject = relationship('Subject')

    # Тип дисциплины
    subject_type = Column(Integer)

    # Наименование дисциплины
    subject_name_ru = Column(String(500))
    subject_name_kz = Column(String(500))
    subject_name_en = Column(String(500))

    # Тип оценки
    mark_type_id = Column(ForeignKey('univer_mark_type.mark_type_id'))
    mark_type = relationship('MarkType')

    # Кредит
    credit = Column('progress_credit', Integer)

    # Оценки
    result_rk1 = Column('progress_result_rk1', Integer)
    result_rk2 = Column('progress_result_rk2', Integer)
    result = Column('progress_result', Integer)

    # Семестры
    semester = Column('n_seme', Integer)

    # Тип контроля
    controll_type_id = Column(ForeignKey(
        'univer_controll_type.controll_type_id'))
    controll_type = relationship('ControllType')

    # Часы
    hours = Column('progress_hours', Integer)

    # Статус
    status = Column(Integer)

    # Модуль
    acpos_module = Column(Integer)

    def __repr__(self):
        return '<Progress {}>'.format(self)

    def __str__(self):
        return '{} - {}'.format(self.student, self.subject)


class Country(Base):
    __tablename__ = 'univer_country'

    id = Column('country_id', Integer, primary_key=True)
    status = Column(Integer)
    name_ru = Column('country_name_ru', String(500))
    name_kz = Column('country_name_kz', String(500))
    name_en = Column('country_name_en', String(500))
    code = Column('country_code', Integer)
    current = Column('country_current', Integer)
    letter_code = Column('country_letter_code', String(10))
    alfa3_code = Column(String(5))

    def __repr__(self):
        return '<Country {}>'.format(self)

    def __str__(self):
        return self.name_ru


class EduStatus(Base):
    """
    Статус студента
    """
    __tablename__ = 'univer_student_edu_statuses'

    # Идентификатор
    id = Column('student_edu_status_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('student_edu_status_name_ru', String(100))
    name_kz = Column('student_edu_status_name_kz', String(100))
    name_en = Column('student_edu_status_name_en', String(100))

    # Статус
    # Возможные значения:
    # - 1 (Активный)
    # - 2 (Архивный)
    status = Column(Integer)

    @validates('status')
    def validate_status(self, key, value):
        assert value in [1, 2]
        return value

    # Статус для студента
    # Возможные значения:
    # - 1 (Активный)
    # - 2 (Архивный)
    # - 10 (Абитуриент)
    status_for_student = Column(Integer)

    @validates('status_for_student')
    def validate_status_for_student(self, key, value):
        assert value in [1, 2, 10]
        return value

    def __repr__(self):
        return f'<EduStatus {self} (id={self.id} status={self.status})>'

    def __str__(self):
        return f'{self.name_ru}'


class SettlementStatus(Base):
    """
    Неизвестный модель
    """
    __tablename__ = 'univer_settlement_status'

    # Идентификатор
    id = Column('settlement_status_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('settlement_status_name_ru', String(100))
    name_kz = Column('settlement_status_name_kz', String(100))
    name_en = Column('settlement_status_name_en', String(100))

    # Статус
    status = Column(Integer)

    @validates('status')
    def validate_status(self, key, value):
        """
        Значением status могут быть следующие значения:
        - 1 (Активный)
        - 2 (Архивный)
        """
        assert value in [1, 2]
        return value

    def __repr__(self):
        return f'<SettlementStatus {self} (id={self.id} status={self.status})>'

    def __str__(self):
        return f'{self.name_ru}'


class ForeignLanguage(Base):
    """
    Иностранный язык
    """
    __tablename__ = 'univer_foreign_language'

    # Идентификатор
    id = Column('foreign_language_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('foreign_language_name_ru', String(100))
    name_kz = Column('foreign_language_name_kz', String(100))
    name_en = Column('foreign_language_name_en', String(100))

    # Статус
    status = Column(Integer)

    @validates('status')
    def validate_status(self, key, value):
        """
        Значением status могут быть следующие значения:
        - 1 (Активный)
        - 2 (Архивный)
        """
        assert value in [1, 2]
        return value

    def __repr__(self):
        return f'<ForeignLanguage {self} (id={self.id} status={self.status})>'

    def __str__(self):
        return f'{self.name_ru}'


class AdmissionReason(Base):
    """
    Причина поступления
    """
    __tablename__ = 'univer_admission_reason'

    # Идентификатор
    id = Column('admission_reason_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('admission_reason_name_ru', String(200))
    name_kz = Column('admission_reason_name_kz', String(200))
    name_en = Column('admission_reason_name_en', String(200))

    # Статус
    status = Column(Integer)

    @validates('status')
    def validate_status(self, key, value):
        """
        Значением status могут быть следующие значения:
        - 1 (Активный)
        - 2 (Архивный)
        """
        assert value in [1, 2]
        return value

    def __repr__(self):
        return f'<AdmissionReason {self} (id={self.id} status={self.status})>'

    def __str__(self):
        return f'{self.name_ru}'


class Region(Base):
    """
    Регион
    """
    __tablename__ = 'univer_region'

    # Идентификатор
    id = Column('region_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('region_name_ru', String(100))
    name_kz = Column('region_name_kz', String(100))
    name_en = Column('region_name_en', String(100))

    # Короткое наименование
    short_name_ru = Column('region_short_name_ru', String(100))
    short_name_kz = Column('region_short_name_kz', String(100))
    short_name_en = Column('region_short_name_en', String(100))

    # Код региона
    code = Column('region_code', Integer)

    # Текущий регион
    current = Column('current_region', Integer)

    @validates('current')
    def validate_current(self, key, value):
        """
        Значением current могут быть следующие значения:
        - 0 (Не текущий)
        - 1 (Текущий)
        """
        assert value in [0, 1]
        return value

    # Статус
    status = Column(Integer)

    @validates('status')
    def valdate_status(self, key, value):
        """
        Значением status могут быть следующие значения:
        - 1 (Активный)
        - 2 (Архивный)
        """
        assert value in [1, 2]
        return value

    def __repr__(self):
        return f'<Region {self} (id={self.id} status={self.status} current={self.current})>'

    def __str__(self):
        return f'{self.name_ru}'


class District(Base):
    """
    Район
    """
    __tablename__ = 'univer_district'

    # Идентификатор
    id = Column('district_id', Integer, primary_key=True)

    # Статус
    status = Column(Integer)

    @validates('status')
    def validate_status(self, key, value):
        """
        Значением status могут быть следующие значения:
        - 1 (Активный)
        - 2 (Архивный)
        """
        assert value in [1, 2]
        return value

    # Регион
    region_id = Column(ForeignKey('univer_region.region_id'))
    region = relationship('Region')

    # Наименование
    name_ru = Column('district_name_ru', String())
    name_kz = Column('district_name_kz', String())
    name_en = Column('district_name_en', String())

    # Код района
    code = Column('district_code', Integer)

    # Текущий район
    current = Column('current_district', Integer)

    @validates('current')
    def validate_current(self, key, value):
        """
        Значением current могут быть следующие значения:
        - 0 (Не текущий)
        - 1 (Текущий)
        """
        assert value in [0, 1]
        return value

    def __repr__(self):
        return f'<District {self} (id={self.id} status={self.status} current={self.current})>'

    def __str__(self):
        return f'{self.name_ru}'


class EducPlanPosGroup(Base):
    """
    Группа позиции учебного плана
    """
    __tablename__ = 'univer_educ_plan_pos_groups'

    # Идентификатор
    id = Column('pos_group_id', Integer, primary_key=True)

    # Учебный план
    educ_plan_id = Column(ForeignKey('univer_educ_plan.educ_plan_id'))
    educ_plan = relationship('EducPlan')

    # Семестр позиции учебного плана
    educ_plan_pos_semester = Column('educ_plan_pos_semestr', Integer)

    # Наименование
    name_ru = Column('pos_group_name_ru', String)
    name_kz = Column('pos_group_name_kz', String)
    name_en = Column('pos_group_name_en', String)

    def __repr__(self):
        return f'<EducPlanPosGroup {self} (id={self.id} educ_plan_id={self.educ_plan_id})>'

    def __str__(self):
        return self.name_ru



import unittest
from univer_db.models import TeacherChairLink, OfficeRegistrator, OfficeRegistratorFacultyLink, Adviser

from . import Session


class TestTeacherChairLink(unittest.TestCase):
    def test_get_obj(self):
        session = Session()
        links = session.query(TeacherChairLink).filter()
        print("links: %s" % links.count())
        self.assertTrue(links.count())


class TestOfficeRegistrator(unittest.TestCase):
    def test_get_obj(self):
        session = Session()
        office_registrator = session.query(OfficeRegistrator).first()
        print(f'office_registrator: {office_registrator}')

        for faculty_links in office_registrator.faculties:
            print(f'faculty: {faculty_links.faculty}')


class TestAdviser(unittest.TestCase):
    def test_get_obj(self):
        session = Session()
        adviser = session.query(Adviser).filter(Adviser.status == 1).first()
        print(f'adviser: {adviser}')

        total = adviser.students.count()

        for index, adviser_student in enumerate(adviser.students):
            print(f'[{index + 1}/{total}] student: {adviser_student.student}')



import os

from univer_db.config import Config
from univer_db.orm import get_session


config = Config(
    host=os.environ.get('UNIVER_HOST'),
    user=os.environ.get('UNIVER_USER'),
    password=os.environ.get('UNIVER_PASSWORD')
)

Session = get_session(config)



from sqlalchemy import Column, Integer, String, Float, Boolean, ForeignKey, Date, DateTime, Time
from sqlalchemy.orm import relationship

from univer_db.orm import get_base


Base = get_base()


class ScheduleTimeType(Base):
    """
    Модель "Тип времени расписания"
    """
    __tablename__ = 'univer_schedule_time_type'

    # Идентификтор
    id = Column('schedule_time_type_id', Integer, primary_key=True)

    # Наименование (на казахском)
    name_kz = Column('schedule_time_type_name_kz', String)

    # Наименование (на русском)
    name_ru = Column('schedule_time_type_name_ru', String)

    # Наименование (на английском)
    name_en = Column('schedule_time_type_name_en', String)

    def __repr__(self):
        return '<ScheduleTimeType {}>'.format(self)

    def __str__(self):
        return str(self.id)


class ScheduleTime(Base):
    """
    Модель "Время расписания
    """
    __tablename__ = 'univer_schedule_time'

    # Идентификатор
    id = Column('schedule_time_id', Integer, primary_key=True)

    # Время начало
    begin = Column('schedule_time_begin', Time)

    # Время окончания
    end = Column('schedule_time_end', Time)

    # Статус
    status = Column(Integer)

    # Тип времени расписания
    type_id = Column('schedule_time_type_id', ForeignKey(
        'univer_schedule_time_type.schedule_time_type_id'))
    type = relationship('ScheduleTimeType')

    def __repr__(self):
        return '<ScheduleTime {}>'.format(self)

    def __str__(self):
        return str(self.id)


class Schedule(Base):
    """
    Модель "Расписание"
    """
    __tablename__ = 'univer_schedule'

    # Идентификатор
    id = Column('schedule_id', Integer, primary_key=True)

    # Группа
    group_id = Column(ForeignKey('univer_group.group_id'))
    group = relationship('Group')

    # Время расписаниея
    time_id = Column('schedule_time_id', ForeignKey(
        'univer_schedule_time.schedule_time_id'))
    time = relationship('ScheduleTime')

    # День недели
    week_day = Column('schedule_week_day', Integer)

    # Аудитория
    audience_id = Column(ForeignKey('univer_audience.audience_id'))
    audience = relationship('Audience')

    def __repr__(self):
        return '<Schedule {}>'.format(self)

    def __str__(self):
        return str(self.id)


class ExamSchedule(Base):
    """
    Модель "Расписание экзамена"
    """
    __tablename__ = 'univer_exam_schedule'

    # Идентификатор
    id = Column('exam_schedule_id', Integer, primary_key=True)

    # Группа
    group_id = Column(ForeignKey('univer_group.group_id'))
    group = relationship('Group')

    # Экзаменатор
    examiner_id = Column('examiner_teacher_id',
                         ForeignKey('univer_teacher.teacher_id'))
    examiner = relationship('Teacher')

    # Аудитория
    audience_id = Column(ForeignKey('univer_audience.audience_id'))
    audience = relationship('Audience')

    # Дата и время экзамена
    exam_time = Column(DateTime)

    # Время экзамена
    time_id = Column('schedule_time_id', ForeignKey(
        'univer_exam_schedule_time.exam_schedule_time_id'))
    time = relationship('ExamScheduleTime')

    def __repr__(self):
        return '<ExamSchedule {}>'.format(self)

    def __str__(self):
        return str(self.id)


class ExamScheduleTime(Base):
    """
    Модель "Время расписания экзамена"
    """
    __tablename__ = 'univer_exam_schedule_time'

    # Идентификатор
    id = Column('exam_schedule_time_id', Integer, primary_key=True)

    # Время начала экзамена
    begin = Column(Time)

    # Тип (?)
    type = Column(Integer)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<ExamScheduleTime {}>'.format(self)

    def __str__(self):
        return str(self.id)



from sqlalchemy import Column, Integer, String, Float, Boolean, ForeignKey, Date, DateTime, REAL
from sqlalchemy.orm import relationship

from univer_db.orm import get_base


Base = get_base()


class Sheet(Base):
    """
    Модель "Ведомость"
    """

    __tablename__ = 'univer_sheet'

    # Идентификатор
    id = Column('sheet_id', Integer, primary_key=True)

    # Статус
    status = Column(Integer)

    # Тип ведомости
    sheet_type_id = Column(ForeignKey('univer_sheet_type.sheet_type_id'))
    sheet_type = relationship('SheetType')

    # Дата создания
    date_create = Column(DateTime)

    # Дата и время закрытия
    date_keep = Column(DateTime)

    # Дата проведения
    date_control = Column(DateTime)

    # Группа
    group_id = Column(ForeignKey('univer_group.group_id'))
    group = relationship('Group')

    # Дата и время изменения
    updated_at = Column(DateTime)

    def __repr__(self):
        return '<Sheet {}>'.format(self)

    def __str__(self):
        return str(self.id)


class SheetType(Base):
    """
    Тип ведомости
    """
    __tablename__ = 'univer_sheet_type'

    # Идентификатор
    id = Column('sheet_type_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('sheet_type_name_ru', String(200))
    name_kz = Column('sheet_type_name_kz', String(200))
    name_en = Column('sheet_type_name_en', String(200))

    # Короткое наименование
    short_name_ru = Column('sheet_type_short_name_ru', String)
    short_name_kz = Column('sheet_type_short_name_kz', String)
    short_name_en = Column('sheet_type_short_name_en', String)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<SheetType {} (id={} status={})>'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class SheetResult(Base):
    """
    Результат ведомости
    """
    __tablename__ = 'univer_sheet_result'

    # Ведомость
    sheet_id = Column(ForeignKey('univer_sheet.sheet_id'), primary_key=True)
    sheet = relationship('Sheet')

    # Дисциплина
    subject_id = Column(ForeignKey('univer_subject.subject_id'))
    subject = relationship('Subject')

    # Преподаватель
    teacher_id = Column(ForeignKey('univer_teacher.teacher_id'))
    teacher = relationship('Teacher')

    # Академический год
    academ_year = Column(Integer)

    # Семестр академического кода
    semester = Column('semestr', Integer)

    # Контроль
    control_id = Column('control', ForeignKey(
        'univer_control.control_id'), primary_key=True)
    control = relationship('Controll')

    # Студент
    student_id = Column(ForeignKey(
        'univer_students.students_id'), primary_key=True)
    student = relationship('Student')

    # Результат
    result = Column(REAL)

    # Неизвестное поле
    date_keep = Column(DateTime)

    # Неизвестное поле
    P_P = Column(Integer)

    # Общий семестр
    n_seme = Column(Integer)

    # Неизвестное поле
    mark_sheet_result = Column(Integer)

    # Неизвестное поле
    retake_type = Column(Integer)

    def __repr__(self):
        return f'<SheetResult {self} (result={self.result})>'

    def __str__(self):
        return f'{self.sheet_id}'



from sqlalchemy import Column, Integer, String, Float, Boolean, ForeignKey, Date, DateTime
from sqlalchemy.orm import relationship

from univer_db.orm import get_base


Base = get_base()


class PaymentForm(Base):
    """
    Модель "Форма оплаты"
    Статус: Выполняется
    """

    __tablename__ = 'univer_payment_forms'

    # Идентификатор
    id = Column('payment_form_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('payment_form_name_ru', String(100))
    name_kz = Column('payment_form_name_kz', String(100))
    name_en = Column('payment_form_name_en', String(100))

    # Краткое наименование
    short_name_ru = Column('payment_form_short_name_ru', String(100))
    short_name_kz = Column('payment_form_short_name_kz', String(100))
    short_name_en = Column('payment_form_short_name_en', String(100))

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<PaymentForm {} (id={} status={})>'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class Stage(Base):
    """
    Модель "Ступень обучения"
    Статус: Выполняется
    """

    __tablename__ = 'univer_stage'

    # Идентификатор
    id = Column('stage_id', Integer, primary_key=True)

    # Статус
    status = Column(Integer)

    # Наименование
    name_kz = Column('stage_name_kz', String(200))
    name_ru = Column('stage_name_ru', String(200))
    name_en = Column('stage_name_en', String(200))

    def __repr__(self):
        return "<Stage {} (id={} status={})>".format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class EducationForm(Base):
    """
    Модель "Форма обучения"
    Статус: Выполняется
    """

    __tablename__ = 'univer_education_form'

    # Идентификатор
    id = Column('education_form_id', Integer, primary_key=True)

    # Статус
    status = Column(Integer)

    # Наименование
    name_kz = Column('education_form_name_kz', String(200))
    name_ru = Column('education_form_name_ru', String(200))
    name_en = Column('education_form_name_en', String(200))

    def __repr__(self):
        return "<EducationForm {} (id={} status={})>".format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class EduLevel(Base):
    """
    Модель "Уровень обучения"
    Статус: Выполняется
    """

    __tablename__ = 'univer_edu_levels'

    # Идентификатор
    id = Column('edu_level_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('edu_level_name_ru', String(100))
    name_kz = Column('edu_level_name_kz', String(100))
    name_en = Column('edu_level_name_en', String(100))

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<EduLevel {} (id={} status={})'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class EnrollmentType(Base):
    """
    Модель "Тип поступления"
    Статус: Выполняется
    """

    __tablename__ = 'univer_enrollment_type'

    # Идентификатор
    id = Column('enrollment_type_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('enrollment_type_name_ru', String(100))
    name_kz = Column('enrollment_type_name_kz', String(100))
    name_en = Column('enrollment_type_name_en', String(100))

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<EnrollmentType {} (id={} status={})>'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class Audience(Base):
    """
    Модель: Аудитория
    """

    __tablename__ = 'univer_audience'

    # Идентификатор
    id = Column('audience_id', Integer, primary_key=True)

    # Номер аудитории (На русском)
    number_ru = Column('audience_number_ru', String)

    # Номер аудитории (На казахском)
    number_kz = Column('audience_number_kz', String)

    # Номер аудитории (На английском))
    number_en = Column('audience_number_en', String)


class Citizenship(Base):
    """
    Гражданство
    """
    __tablename__ = 'univer_citizenship_1c'

    # Идентификатор
    id = Column('citizenship_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('citizenship_name_ru', String(500))
    name_kz = Column('citizenship_name_kz', String(500))
    name_en = Column('citizenship_name_en', String(500))

    # Короткое наименование
    short_name_ru = Column('citizenship_short_name_ru', String(500))
    short_name_kz = Column('citizenship_short_name_kz', String(500))
    short_name_en = Column('citizenship_short_name_en', String(500))

    # ISO-коды
    code = Column('citizenship_code', Integer)
    code2 = Column('citizenship_code2', String(250))
    code3 = Column('citizenship_code3', String(250))

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<Citizenship {} (id={} code2={} status={})>'.format(self, self.id, self.code2, self.status)

    def __str__(self):
        return self.name_ru


class Nationality(Base):
    """
    Национальность
    """
    __tablename__ = 'univer_nationality'

    # Идентификатор
    id = Column('nationality_id', Integer, primary_key=True)

    # Статус
    status = Column(Integer)

    # Наименование
    name_ru = Column('nationality_name_ru', String(100))
    name_kz = Column('nationality_name_kz', String(100))
    name_en = Column('nationality_name_en', String(100))

    # Короткое наименование
    short_name_ru = Column('nationality_short_name_ru', String(20))
    short_name_kz = Column('nationality_short_name_kz', String(20))
    short_name_en = Column('nationality_short_name_en', String(20))

    def __repr__(self):
        return '<Nationality {} (id={} status={})>'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class Grant(Base):
    """
    Грант студента
    """
    __tablename__ = 'univer_grant'

    # Идентификатор
    id = Column('grant_id', Integer, primary_key=True)

    # Дата назначения гранта
    date_recieved = Column('grant_date_recieved', DateTime)

    # Номер гранта
    number = Column('grant_number', String(15))

    # Студент
    student_id = Column(ForeignKey('univer_students.students_id'))
    student = relationship('Student')

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<Grant {} (id={} student={} status={})>'.format(self, self.id, self.student_id, self.status)

    def __str__(self):
        return f'Грант - {self.student}: {self.number} ({self.date_recieved})'


class Contract(Base):
    """
    Договор студента
    """
    __tablename__ = 'univer_contract'

    # Идентификатор
    id = Column('contract_id', Integer, primary_key=True)

    # Номер договора
    number = Column('contract_number', String(50))

    # Дата начало договора
    date_recieved = Column('contract_date_recieved', DateTime)

    # Студент
    student_id = Column('students_id', ForeignKey(
        'univer_students.students_id'))
    student = relationship('Student')

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<Contract {} (id={} student={} status={})>'.format(self, self.id, self.student_id, self.status)

    def __str__(self):
        return f'Договор - {self.student}: {self.number} ({self.date_recieved})'


class FamilyStatus(Base):
    """
    Семейное положение
    """
    __tablename__ = 'univer_family_status_1c'

    # Идентификатор
    id = Column('family_status_id', Integer, primary_key=True)

    # Наименование
    name_ru = Column('family_status_name_ru', String(500))
    name_kz = Column('family_status_name_kz', String(500))
    name_en = Column('family_status_name_en', String(500))

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return '<FamilyStatus {} (id={} status={})>'.format(self, self.id, self.status)

    def __str__(self):
        return self.name_ru


class Alert(Base):
    """
    Таблица системных сообщений
    """
    __tablename__ = 'univer_alert'

    # Пользователь
    user_id = Column(ForeignKey('univer_users.user_id'), primary_key=True)
    user = relationship('User')

    # Код сообщений
    code = Column('alert_code', Integer, primary_key=True)

    # Дата сообщений
    date = Column('alert_date', DateTime, primary_key=True)

    def __repr__(self):
        return f'<Alert (user_id={self.user_id} code={self.code})'

    def __str__(self):
        return f'{self.user_id}: {self.code}'



from .models import *
from .base import *
from .roles import *
from .orders import *
from .dormitories import *
from .structures import *
from .sheets import *
from .grades import *
from .schedule import *
from .geo import *



import urllib
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()


def get_session(config):
    quoted = urllib.parse.quote_plus('DRIVER={driver};Server={host};Database={db};UID={user};PWD={password};TDS_Version=7.3;Port=1433;'.format(
        driver=config.driver, host=config.host, db=config.db, user=config.user, password=config.password))
    engine = create_engine('mssql+pyodbc:///?odbc_connect={}'.format(quoted),
                           pool_size=config.pool_size, max_overflow=config.max_overflow)

    engine.connect()

    session_factory = sessionmaker(bind=engine)

    return session_factory


def get_base():
    return Base



"""
Роли и пользовали внутри Univer
"""

from sqlalchemy import Column, Integer, String, ForeignKey, DateTime
from sqlalchemy.orm import relationship, backref, validates

from univer_db.orm import get_base


Base = get_base()


class User(Base):
    """
    Модель "Пользователи"
    Статус: Выполняется
    """

    __tablename__ = 'univer_users'

    # Идентификатор
    id = Column('user_id', Integer, primary_key=True)

    # Логин
    username = Column('user_login', String(50))

    # Пароль
    password = Column('user_password', String(128))

    # Временный пароль
    temppass = Column('user_temppass', String(50))

    # Права доступа пользователя
    access = Column('user_access', Integer, default=0)

    def __repr__(self):
        return f'<User {self} (id={self.id})>'

    def __str__(self):
        return self.username


class Student(Base):
    """
    Студенты
    """
    __tablename__ = 'univer_students'
    __table_args__ = {
        'implicit_returning': False
    }

    # Идентификатор
    id = Column('students_id', Integer, primary_key=True)

    # Статус
    status = Column('status', Integer)

    @validates('status')
    def validate_status(self, key, value):  # pylint: disable=unused-argument
        """
        Значением status могут быть следующие значения:
        - 1 (Активный)
        - 2 (Архивный)
        - 10 (Абитуриент)
        """
        assert value in [1, 2, 10]
        return value

    # Пользователь
    user_id = Column('user_id', ForeignKey('univer_users.user_id'))
    user = relationship('User')

    # Ступень обучения
    stage_id = Column(ForeignKey('univer_stage.stage_id'))
    stage = relationship('Stage')

    # Факультет
    faculty_id = Column(ForeignKey('univer_faculty.faculty_id'))
    faculty = relationship('Faculty')

    # Специальность/ОП
    speciality_id = Column(ForeignKey('univer_speciality.speciality_id'))
    speciality = relationship('Speciality')

    # Страна
    country_id = Column(ForeignKey('univer_country.country_id'))
    country = relationship('Country')

    # Дата регистрации
    reg_date = Column('student_reg_date', DateTime)

    # Уровень обучения
    edu_level_id = Column('edu_levels_id', ForeignKey(
        'univer_edu_levels.edu_level_id'))
    edu_level = relationship('EduLevel')

    # Национальность
    nationality_id = Column(ForeignKey('univer_nationality.nationality_id'))
    nationality = relationship('Nationality')

    # Гражданство
    citizenship_id = Column(ForeignKey('univer_citizenship_1c.citizenship_id'))
    citizenship = relationship('Citizenship')

    # Тип оплаты
    payment_form_id = Column('payment_forms_id', ForeignKey(
        'univer_payment_forms.payment_form_id'))
    payment_form = relationship('PaymentForm')

    # Форма обучения
    education_form_id = Column(ForeignKey(
        'univer_education_form.education_form_id'))
    education_form = relationship('EducationForm')

    # Неизвестное поле
    settlement_status_id = Column(ForeignKey(
        'univer_settlement_status.settlement_status_id'))
    settlement_status = relationship('SettlementStatus')

    # Иностранный язык
    foreign_language_id = Column(ForeignKey(
        'univer_foreign_language.foreign_language_id'))
    foreign_language = relationship('ForeignLanguage')

    # Причина поступления
    admission_reason_id = Column(ForeignKey(
        'univer_admission_reason.admission_reason_id'))
    admission_reason = relationship('AdmissionReason')

    # Регион
    region_id = Column(ForeignKey('univer_region.region_id'))
    region = relationship('Region')

    # Тип поступления
    enrollment_type_id = Column(ForeignKey(
        'univer_enrollment_type.enrollment_type_id'))
    enrollment_type = relationship('EnrollmentType')

    # Район
    district_id = Column(ForeignKey('univer_district.district_id'))
    district = relationship('District')

    # ФИО студента
    last_name = Column('students_sname', String(100))
    first_name = Column('students_name', String(100))
    middle_name = Column('students_father_name', String(100))

    # Дата рождения
    birth_date = Column('students_birth_date', DateTime)

    # Электронная почта
    email = Column('students_email', String(25))

    # Пол
    sex = Column('students_sex', Integer)

    @validates('sex')
    def validate_sex(self, key, value):  # pylint: disable=unused-argument
        """
        Значением sex могут быть следующие значения:
        - 1 (Мужчина)
        - 0 (Женщина)
        """
        assert value in [0, 1]
        return value

    marital_status = Column('students_marital_status', Integer)

    @validates('marital_status')
    def validate_marital_status(self, key, value):  # pylint: disable=unused-argument
        """
        Значением marital_status могут быть следующие значения:
        - 0 (Не женат/Не замужена)
        - 1 (Женат/замужена)
        """
        assert value in [0, 1]
        return value

    # Статус студента
    edu_status_id = Column('student_edu_status_id', ForeignKey(
        'univer_student_edu_statuses.student_edu_status_id'))
    edu_status = relationship('EduStatus')

    # Мобильный телефон
    mobile_phone = Column('students_mobile_phone', String(50))

    # ФИО студента в дательном падеже (На русском языке)
    dative_last_name_ru = Column('students_dative_sname_ru', String(100))
    dative_first_name_ru = Column('students_dative_name_ru', String(100))
    dative_middle_name_ru = Column(
        'students_dative_father_name_ru', String(100))

    # ФИО студента в дательном падеже (На казахском языке)
    dative_last_name_kz = Column('students_dative_sname_kz', String(100))
    dative_first_name_kz = Column('students_dative_name_kz', String(100))
    dative_middle_name_kz = Column(
        'students_dative_father_name_kz', String(100))

    # ФИО студента в дательном падеже (На английском языке)
    dative_last_name_en = Column('students_dative_sname_en', String(100))
    dative_first_name_en = Column('students_dative_name_en', String(100))
    dative_middle_name_en = Column(
        'students_dative_father_name_en', String(100))

    # Фамилия и Имя студента транслитом
    last_name_translit = Column('students_sname_intern', String(100))
    first_name_translit = Column('students_name_intern', String(100))

    # Курс
    course = Column('students_curce_number', Integer)

    # Документ
    document_identity_type_id = Column('students_document_identity_type', ForeignKey(
        'univer_document_identity.document_identity_type'))
    document_identity_type = relationship('DocumentIdentity')
    document_identity_number = Column(
        'students_document_identity_number', String(50))
    document_identity_date = Column(
        'students_document_identity_date', DateTime)
    document_identity_issued = Column(
        'students_document_identity_issued', String(100))

    # ИИН студента
    identify_code = Column('students_identify_code', String(50))

    # Данные об окончании учебного заведения перед поступлением в университет
    graduate_info_id = Column(ForeignKey(
        'univer_graduate_info.graduate_info_id'))
    graduate_info = relationship('GraduateInfo')

    # Год начала действия учебного плана
    educ_plan_adm_year = Column(Integer)

    # Языковое отделение
    lang_division_id = Column(ForeignKey(
        'univer_lang_division.lang_division_id'))
    lang_division = relationship('LangDivision')

    # Нуждается в общежитии
    need_hostel = Column('students_need_hostel', Integer)

    @property
    def payment_info_ru(self):
        """
        Возвращает специальный текст на русском для формы оплаты
        """
        if self.payment_form_id == 2:
            return 'на платной основе'
        elif self.payment_form_id == 5:
            return 'на основе государственного образовательного гранта'

    @property
    def payment_info_kz(self):
        """
        Возвращает специальный текст на казахском для формы оплаты
        """
        if self.payment_form_id == 2:
            return 'ақылы негізде'
        elif self.payment_form_id == 5:
            return 'мемлекеттік білім беру гранты негізінде'

    @property
    def edu_level_info_ru(self):
        """
        Возвращает специальный текст на русском для уровня образования
        """
        if self.edu_level_id == 1:
            return '(бакалавриат, 4 года)'
        elif self.edu_level_id == 3:
            return 'по сокращенной образовательной программе на ' \
                'базе среднего профессионального образования'
        elif self.edu_level_id == 2:
            return 'по сокращенной образовательной программе на базе высшего образования'

    @property
    def edu_level_info_kz(self):
        """
        Возвращает специальный текст на казахском для уровня образования
        """
        if self.edu_level_id == 1:
            return '(бакалавриат, 4 жыл)'
        elif self.edu_level_id == 3:
            return 'орта кәсіптік білім негізінде қысқартылған білім беру ' \
                'бағдарламасы бойынша күндізгі білім беру нысаны'
        elif self.edu_level_id == 2:
            return 'жоғары білім негізінде қысқартылған білім беру ' \
                'бағдарламасы бойынша күндізгі білім беру нысаны'

    @property
    def dative_full_name_ru(self):
        """
        Возвращает ФИО в дательном падеже (На русском)
        """
        last_name = self.dative_last_name_ru if self.dative_last_name_ru else self.last_name
        first_name = self.dative_first_name_ru if self.dative_first_name_ru else self.first_name
        middle_name = self.dative_middle_name_ru if self.dative_middle_name_ru else self.middle_name

        return ' '.join(filter(None, [last_name, first_name, middle_name]))

    @property
    def dative_full_name_kz(self):
        """
        Возвращает ФИО в дательном падеже (На казахском)
        """
        last_name = self.dative_last_name_kz if self.dative_last_name_kz else self.last_name
        first_name = self.dative_first_name_kz if self.dative_first_name_kz else self.first_name
        middle_name = self.dative_middle_name_kz if self.dative_middle_name_kz else self.middle_name

        return ' '.join(
            filter(
                None,
                [last_name, first_name, middle_name]
            )
        )

    @property
    def dative_full_name_en(self):
        """
        Возвращает ФИО в дательном падеже (На английском)
        """
        return ' '.join(
            filter(
                None,
                [
                    self.dative_last_name_en,
                    self.dative_first_name_en,
                    self.dative_middle_name_en
                ]
            )
        )

    def __repr__(self):
        return f'<Student {self} (id={self.id} user={self.user_id} status={self.status})>'

    def __str__(self):
        full_name = ' '.join(
            filter(None, [self.last_name, self.first_name, self.middle_name]))
        return f'{full_name}'


class Personnel(Base):
    """
    Сотрудник
    """
    __tablename__ = 'univer_personal'

    # Идентификатор
    id = Column('personal_id', Integer, primary_key=True)

    # Пользователь
    user_id = Column('user_id', ForeignKey('univer_users.user_id'))
    user = relationship('User')

    # Статус
    status = Column('status', Integer)

    # ФИО персонала
    last_name = Column('personal_sname', String(200))
    first_name = Column('personal_name', String(100))
    middle_name = Column('personal_father_name', String(100))

    # Дата рождения
    birthdate = Column('personal_birthday_date', DateTime)

    # Адрес проживания
    address = Column('personal_adress', String(250))

    # Рабочий адрес
    work_address = Column('personal_work_adress', String(250))

    # Рабочая электронная почта
    work_email = Column('personal_work_email', String(50))

    # Электронная почта
    email = Column('personal_email', String(50))

    # Рабочий телефон
    work_phone = Column('personal_work_phone', String(50))

    # Мобильный телефон
    mobile_phone = Column('personal_mobile_phone', String(50))

    # Домашний телефон
    home_phone = Column('personal_home_phone', String(50))

    # Дата начала работы в ВУЗе
    employment_date = Column('personal_employment_date', DateTime)

    # Пол
    sex = Column('personal_sex', Integer)

    # Фамилия и Имя персонала транслитом
    last_name_translit = Column('personal_translit_sname', String(100))
    first_name_translit = Column('personal_translit_name', String(100))
    middle_name_translit = Column('personal_translit_fname', String(100))

    # ИИН сотрудника
    identify_code = Column('personal_identification_number', String(150))

    # Серия документа
    document_series = Column('personal_document_series', String(150))

    # Номер документа
    document_number = Column('personal_document_number', String(150))

    # Дата выдачи документа
    document_issue_date = Column('personal_document_issue_date', DateTime)

    # Кем выдан документ
    document_issued = Column('personal_document_issued', String(150))

    # Национальность
    nationality_id = Column(ForeignKey('univer_nationality.nationality_id'))
    nationality = relationship('Nationality')

    # Семейное положение
    family_status_id = Column(ForeignKey(
        'univer_family_status_1c.family_status_id'))
    family_status = relationship('FamilyStatus')

    # Гражданство
    citizenship_id = Column(ForeignKey('univer_citizenship_1c.citizenship_id'))
    citizenship = relationship('Citizenship')

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return f'<Personnel {self} (id={self.id} user={self.user_id} status={self.status})>'

    def __str__(self):
        return ' '.join(filter(None, [self.last_name, self.first_name, self.middle_name]))


class Teacher(Base):
    """
    Модель "Преподаватели"
    Статус: Выполняется
    """

    __tablename__ = 'univer_teacher'

    # Идентификатор
    id = Column('teacher_id', Integer, primary_key=True)

    # Сотрудник
    personnel_id = Column('personal_id', ForeignKey(
        'univer_personal.personal_id'))
    personnel = relationship('Personnel')

    # Статус
    status = Column('status', Integer)

    def __repr__(self):
        return f'<Teacher {self} (id={self.id} personnel={self.personnel_id} status={self.status})>'

    def __str__(self):
        return str(self.personnel)


class OfficeRegistrator(Base):
    """
    Модель "Сотрудник Офис-Регистратора"
    """

    __tablename__ = 'univer_office_registrator'

    # Идентификатор
    id = Column('office_registrator_id', Integer, primary_key=True)

    # Сотрудник
    personnel_id = Column('personal_id', ForeignKey(
        'univer_personal.personal_id'))
    personnel = relationship('Personnel')

    # Факультет
    faculty_id = Column(ForeignKey('univer_faculty.faculty_id'))
    faculty = relationship('Faculty')

    # Статус
    status = Column(Integer)

    # Тип
    type = Column(Integer)

    def __repr__(self):
        return f'<OfficeRegistrator {self} (id={self.id})>'

    def __str__(self):
        return str(self.personnel)


class OfficeRegistratorFacultyLink(Base):
    """
    Связь между Специалистом ОР и Факультетом
    """
    __tablename__ = 'univer_office_registrator_faculty_link'

    # Специалист ОР
    office_registrator_id = Column(ForeignKey(
        'univer_office_registrator.office_registrator_id'), primary_key=True)
    office_registrator = relationship('OfficeRegistrator', backref='faculties')

    # Факультет
    faculty_id = Column(ForeignKey(
        'univer_faculty.faculty_id'), primary_key=True)
    faculty = relationship('Faculty')

    def __repr__(self):
        return f'<OfficeRegistratorFacultyLink {self}>'

    def __str__(self):
        return f'{self.office_registrator}: {self.faculty}'


class ChairHead(Base):
    """
    Заведующий кафедрой
    """
    __tablename__ = 'univer_head_chair'

    # Идентификатор
    id = Column('head_chair_id', Integer, primary_key=True)

    # Сотрудник
    personnel_id = Column('personal_id', ForeignKey(
        'univer_personal.personal_id'))
    personnel = relationship('Personnel')

    # Кафедра
    chair_id = Column(ForeignKey('univer_chair.chair_id'))
    chair = relationship('Chair')

    # Дата начала
    start_date = Column('head_chair_start_date', DateTime)

    # Дата окончания
    end_date = Column('head_chair_end_date', DateTime)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return f'<ChairHead {self} (id={self.id} status={self.status})>'

    def __str__(self):
        return f'{self.personnel}: {self.chair}'


class Dean(Base):
    """
    Модель "Декан"
    """
    __tablename__ = 'univer_dekan'

    # Идентификатор
    id = Column('dekan_id', Integer, primary_key=True)

    # Сотрудник
    personnel_id = Column('personal_id', ForeignKey(
        'univer_personal.personal_id'))
    personnel = relationship('Personnel')

    # Факультет
    faculty_id = Column(ForeignKey('univer_faculty.faculty_id'))
    faculty = relationship('Faculty')

    # Дата начала
    start_date = Column('dekan_start_date', DateTime)

    # Дата окончания
    end_date = Column('dekan_end_date', DateTime)

    # Статус
    status = Column(Integer)

    def __repr__(self):
        return f'<Dean {self} (id={self.id})>'

    def __str__(self):
        return str(self.personnel)


class TeacherChairLink(Base):
    """
    Модель отношение между Teacher (Преподаватель) и Chair (Кафедра)
    """
    __tablename__ = 'univer_teacher_chair_link'

    # Преподаватель
    teacher_id = Column(ForeignKey(
        'univer_teacher.teacher_id'), primary_key=True)
    teacher = relationship('Teacher')

    # Кафедра
    chair_id = Column(ForeignKey('univer_chair.chair_id'), primary_key=True)
    chair = relationship('Chair')

    # Ставка
    rate = Column('stavka', Integer)

    def __repr__(self):
        return f'<TeacherChairLink {self}>'

    def __str__(self):
        return f'{self.chair}: {self.teacher}'


class Adviser(Base):
    """
    Модель "Эдвайзер"
    """
    __tablename__ = 'univer_advicer'

    # Идентификатор
    id = Column('advicer_id', Integer, primary_key=True)

    # Сотрудник
    personnel_id = Column('personal_id', ForeignKey(
        'univer_personal.personal_id'))
    personnel = relationship('Personnel')

    # Факультет
    faculty_id = Column(ForeignKey('univer_faculty.faculty_id'))
    faculty = relationship('Faculty')

    # Статус
    status = Column(Integer)

    # Кафедра
    chair_id = Column(ForeignKey('univer_chair.chair_id'))
    chair = relationship('Chair')

    def __repr__(self):
        return f'<Adviser {self}>'

    def __str__(self):
        return str(self.personnel)


class AdviserStudent(Base):
    """
    Модель "Связь между Adviser (Эдвайзер) и Student (Студент)"
    """
    __tablename__ = 'univer_advicer_student_link'

    # Эдвайзер
    adviser_id = Column('advicer_id', ForeignKey(
        'univer_advicer.advicer_id'), primary_key=True)
    adviser = relationship(
        'Adviser', backref=backref('students', lazy='dynamic'))

    # Студент
    student_id = Column(ForeignKey(
        'univer_students.students_id'), primary_key=True)
    student = relationship(
        'Student', backref=backref('advisers', lazy='dynamic'))

    def __repr__(self):
        return f'<AdviserStudent {self}>'

    def __str__(self):
        return f'{self.adviser}: {self.student}'



from sqlalchemy import Column, Integer, String, ForeignKey
from sqlalchemy.orm import relationship

from univer_db.orm import get_base


Base = get_base()


class AbiturientRegAtsType(Base):
    """
    Тип адреса абитуриента
    """
    __tablename__ = 'abiturient_reg_ats_types'

    # Идентификатор
    id = Column(Integer, primary_key=True)

    # Наименование
    value_ru = Column(String(100))
    value_kz = Column(String(100))
    value_en = Column(String(100))

    # Код
    code = Column(Integer)

    # Актуальный
    actual = Column(Integer)

    def __repr__(self):
        return f'<AbiturientRegAtsType {self} (id={self.id} code={self.code} actual={self.actual})>'
    
    def __str__(self):
        return self.value_ru


class AbiturientRegAts(Base):
    """
    Адрес абитуриента
    """
    __tablename__ = 'abiturient_reg_ats'

    # Идентификатор
    id = Column(Integer, primary_key=True)

    # Родитель
    parent_id = Column(ForeignKey('abiturient_reg_ats.id'))
    parent = relationship('AbiturientRegAts')

    # Родители
    parents = Column(String(100))

    # Тип
    type_id = Column('d_ats_type_id', ForeignKey('abiturient_reg_ats_types.id'))
    type = relationship('AbiturientRegAtsType')

    # Наименование
    name_ru = Column('name_rus', String(240))
    name_kz = Column('name_kaz', String(240))
    name_en = Column('name_eng', String(240))

    # RCO
    rco = Column(String(20))

    # КАТО
    cato = Column(String(20))

    # Актуальный
    actual = Column(Integer)

    def __repr__(self):
        return f'<AbiturientRegAts {self} (id={self.id} parent_id={self.parent_id} type_id={self.type_id} actual={self.actual})>'
    
    def __str__(self):
        return self.name_ru
